@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
@@ -10,9 +10,9 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and Digia. For licensing terms and
+** conditions see http://qt.digia.com/licensing. For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -23,8 +23,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights. These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** $QT_END_LICENSE$
@@ -35,9 +35,7 @@
 #define QXCBCONNECTION_H
 
 #include <xcb/xcb.h>
-#include <xcb/randr.h>
 
-#include "qxcbexport.h"
 #include <QHash>
 #include <QList>
 #include <QMutex>
@@ -69,8 +67,7 @@
 #endif
 #endif
 struct XInput2TouchDeviceData;
-#endif // XCB_USE_XINPUT2
-
+#endif
 struct xcb_randr_get_output_info_reply_t;
 
 //#define Q_XCB_DEBUG
@@ -79,9 +76,7 @@ QT_BEGIN_NAMESPACE
 
 Q_DECLARE_LOGGING_CATEGORY(lcQpaXInput)
 Q_DECLARE_LOGGING_CATEGORY(lcQpaXInputDevices)
-Q_DECLARE_LOGGING_CATEGORY(lcQpaScreen)
 
-class QXcbVirtualDesktop;
 class QXcbScreen;
 class QXcbWindow;
 class QXcbDrag;
@@ -90,7 +85,6 @@ class QXcbClipboard;
 class QXcbWMSupport;
 class QXcbNativeInterface;
 class QXcbSystemTrayTracker;
-class QXcbGlIntegration;
 
 namespace QXcbAtom {
     enum Atom {
@@ -109,7 +103,6 @@ namespace QXcbAtom {
         WM_STATE,
         WM_CHANGE_STATE,
         WM_CLASS,
-        WM_NAME,
 
         // Session management
         WM_CLIENT_LEADER,
@@ -194,7 +187,6 @@ namespace QXcbAtom {
         _KDE_NET_WM_WINDOW_TYPE_OVERRIDE,
 
         _KDE_NET_WM_FRAME_STRUT,
-        _NET_FRAME_EXTENTS,
 
         _NET_STARTUP_INFO,
         _NET_STARTUP_INFO_BEGIN,
@@ -261,7 +253,6 @@ namespace QXcbAtom {
         AbsMTPositionY,
         AbsMTTouchMajor,
         AbsMTTouchMinor,
-        AbsMTOrientation,
         AbsMTPressure,
         AbsMTTrackingID,
         MaxContacts,
@@ -287,8 +278,6 @@ namespace QXcbAtom {
         _COMPIZ_DECOR_PENDING,
         _COMPIZ_DECOR_REQUEST,
         _COMPIZ_DECOR_DELETE_PIXMAP,
-        _COMPIZ_TOOLKIT_ACTION,
-        _GTK_LOAD_ICONTHEMES,
 
         NPredefinedAtoms,
 
@@ -306,7 +295,7 @@ class QXcbEventReader : public QThread
 public:
     QXcbEventReader(QXcbConnection *connection);
 
-    void run() Q_DECL_OVERRIDE;
+    void run();
 
     QXcbEventArray *lock();
     void unlock();
@@ -327,6 +316,9 @@ private:
     QMutex m_mutex;
     QXcbEventArray m_events;
     QXcbConnection *m_connection;
+
+    typedef xcb_generic_event_t * (*XcbPollForQueuedEventFunctionPointer)(xcb_connection_t *c);
+    XcbPollForQueuedEventFunctionPointer m_xcb_poll_for_queued_event;
 };
 
 class QXcbWindowEventListener
@@ -349,10 +341,7 @@ public:
     virtual void handleFocusInEvent(const xcb_focus_in_event_t *) {}
     virtual void handleFocusOutEvent(const xcb_focus_out_event_t *) {}
     virtual void handlePropertyNotifyEvent(const xcb_property_notify_event_t *) {}
-#ifdef XCB_USE_XINPUT22
-    virtual void handleXIMouseEvent(xcb_ge_event_t *, Qt::MouseEventSource = Qt::MouseEventNotSynthesized) {}
-    virtual void handleXIEnterLeave(xcb_ge_event_t *) {}
-#endif
+
     virtual QXcbWindow *toWindow() { return 0; }
 };
 
@@ -371,19 +360,17 @@ private:
 };
 
 class QAbstractEventDispatcher;
-class Q_XCB_EXPORT QXcbConnection : public QObject
+class QXcbConnection : public QObject
 {
     Q_OBJECT
 public:
-    QXcbConnection(QXcbNativeInterface *nativeInterface, bool canGrabServer, xcb_visualid_t defaultVisualId, const char *displayName = 0);
+    QXcbConnection(QXcbNativeInterface *nativeInterface, bool canGrabServer, const char *displayName = 0);
     ~QXcbConnection();
 
     QXcbConnection *connection() const { return const_cast<QXcbConnection *>(this); }
 
-    const QList<QXcbVirtualDesktop *> &virtualDesktops() const { return m_virtualDesktops; }
     const QList<QXcbScreen *> &screens() const { return m_screens; }
     int primaryScreenNumber() const { return m_primaryScreenNumber; }
-    QXcbVirtualDesktop *primaryVirtualDesktop() const { return m_virtualDesktops.value(m_primaryScreenNumber); }
     QXcbScreen *primaryScreen() const;
 
     inline xcb_atom_t atom(QXcbAtom::Atom atom) const { return m_allAtoms[atom]; }
@@ -407,30 +394,26 @@ public:
 
     QXcbWMSupport *wmSupport() const { return m_wmSupport.data(); }
     xcb_window_t rootWindow();
-    xcb_window_t clientLeader();
-
-    bool hasDefaultVisualId() const { return m_defaultVisualId != UINT_MAX; }
-    xcb_visualid_t defaultVisualId() const { return m_defaultVisualId; }
-
 #ifdef XCB_USE_XLIB
-    void *xlib_display() const;
-    void *createVisualInfoForDefaultVisualId() const;
+    void *xlib_display() const { return m_xlib_display; }
 #endif
 
+#ifdef XCB_USE_EGL
+    bool hasEgl() const;
+#endif
+#if defined(XCB_USE_EGL)
+    void *egl_display() const { return m_egl_display; }
+#endif
 #if defined(XCB_USE_XINPUT2)
     void xi2Select(xcb_window_t window);
 #endif
 #ifdef XCB_USE_XINPUT21
-    bool isAtLeastXI21() const { return m_xi2Enabled && m_xi2Minor >= 1; }
+    bool isUsingXInput21() { return m_xi2Enabled && m_xi2Minor >= 1; }
 #else
-    bool isAtLeastXI21() const { return false; }
-#endif
-#ifdef XCB_USE_XINPUT22
-    bool isAtLeastXI22() const { return m_xi2Enabled && m_xi2Minor >= 2; }
-#else
-    bool isAtLeastXI22() const { return false; }
+    bool isUsingXInput21() { return false; }
 #endif
 
+
     void sync();
 
     void handleXcbError(xcb_generic_error_t *error);
@@ -454,29 +437,22 @@ public:
     inline xcb_timestamp_t netWmUserTime() const { return m_netWmUserTime; }
     inline void setNetWmUserTime(xcb_timestamp_t t) { if (t > m_netWmUserTime) m_netWmUserTime = t; }
 
+    bool hasGLX() const { return has_glx_extension; }
     bool hasXFixes() const { return xfixes_first_event > 0; }
     bool hasXShape() const { return has_shape_extension; }
     bool hasXRandr() const { return has_randr_extension; }
     bool hasInputShape() const { return has_input_shape; }
+    bool hasTouchWithoutMouseEmulation() const { return has_touch_without_mouse_emulation; }
     bool hasXKB() const { return has_xkb; }
 
     bool supportsThreadedRendering() const { return m_reader->isRunning(); }
-    bool threadedEventHandling() const { return m_reader->isRunning(); }
 
     xcb_timestamp_t getTimestamp();
-    xcb_window_t getSelectionOwner(xcb_atom_t atom) const;
-    xcb_window_t getQtSelectionOwner();
 
-    void setButton(Qt::MouseButton button, bool down) { if (down) m_buttons |= button; else m_buttons &= ~button; }
     Qt::MouseButtons buttons() const { return m_buttons; }
-    Qt::MouseButton translateMouseButton(xcb_button_t s);
 
     QXcbWindow *focusWindow() const { return m_focusWindow; }
     void setFocusWindow(QXcbWindow *);
-    QXcbWindow *mouseGrabber() const { return m_mouseGrabber; }
-    void setMouseGrabber(QXcbWindow *);
-    QXcbWindow *mousePressWindow() const { return m_mousePressWindow; }
-    void setMousePressWindow(QXcbWindow *);
 
     QByteArray startupId() const { return m_startupId; }
     void setStartupId(const QByteArray &nextId) { m_startupId = nextId; }
@@ -487,29 +463,15 @@ public:
 
     QXcbNativeInterface *nativeInterface() const { return m_nativeInterface; }
 
-    QXcbSystemTrayTracker *systemTrayTracker() const;
-    static bool xEmbedSystemTrayAvailable();
-    static bool xEmbedSystemTrayVisualHasAlphaChannel();
+    QXcbSystemTrayTracker *systemTrayTracker();
 
-#ifdef XCB_USE_XINPUT21
-    void handleEnterEvent();
+#ifdef XCB_USE_XINPUT2
+    void handleEnterEvent(const xcb_enter_notify_event_t *);
 #endif
 
-#ifdef XCB_USE_XINPUT22
-    bool xi2SetMouseGrabEnabled(xcb_window_t w, bool grab);
-#endif
-    Qt::MouseButton xiToQtMouseButton(uint32_t b);
-
     QXcbEventReader *eventReader() const { return m_reader; }
 
     bool canGrab() const { return m_canGrabServer; }
-
-    QXcbGlIntegration *glIntegration() const { return m_glIntegration; }
-
-#ifdef XCB_USE_XINPUT22
-    bool xi2MouseEvents() const;
-#endif
-
 protected:
     bool event(QEvent *e) Q_DECL_OVERRIDE;
 
@@ -522,29 +484,23 @@ private slots:
 private:
     void initializeAllAtoms();
     void sendConnectionEvent(QXcbAtom::Atom atom, uint id = 0);
+    void initializeGLX();
     void initializeXFixes();
     void initializeXRender();
     void initializeXRandr();
-    void initializeXinerama();
     void initializeXShape();
     void initializeXKB();
     void handleClientMessageEvent(const xcb_client_message_event_t *event);
-    QXcbScreen* findScreenForCrtc(xcb_window_t rootWindow, xcb_randr_crtc_t crtc);
-    QXcbScreen* findScreenForOutput(xcb_window_t rootWindow, xcb_randr_output_t output);
-    QXcbVirtualDesktop* virtualDesktopForRootWindow(xcb_window_t rootWindow);
-    void updateScreens(const xcb_randr_notify_event_t *event);
-    bool checkOutputIsPrimary(xcb_window_t rootWindow, xcb_randr_output_t output);
-    void updateScreen(QXcbScreen *screen, const xcb_randr_output_change_t &outputChange);
-    QXcbScreen *createScreen(QXcbVirtualDesktop *virtualDesktop,
-                             const xcb_randr_output_change_t &outputChange,
-                             xcb_randr_get_output_info_reply_t *outputInfo);
-    void destroyScreen(QXcbScreen *screen);
-    void initializeScreens();
-    bool compressEvent(xcb_generic_event_t *event, int currentIndex, QXcbEventArray *eventqueue) const;
+    QXcbScreen* findOrCreateScreen(QList<QXcbScreen *>& newScreens, int screenNumber,
+        xcb_screen_t* xcbScreen, xcb_randr_get_output_info_reply_t *output = NULL);
+    void updateScreens();
+    void handleButtonPress(xcb_generic_event_t *event);
+    void handleButtonRelease(xcb_generic_event_t *event);
+    void handleMotionNotify(xcb_generic_event_t *event);
 
-#ifdef XCB_USE_XINPUT2
     bool m_xi2Enabled;
     int m_xi2Minor;
+#ifdef XCB_USE_XINPUT2
     void initializeXInput2();
     void finalizeXInput2();
     void xi2SetupDevices();
@@ -553,9 +509,6 @@ private:
     void xi2HandleHierachyEvent(void *event);
     void xi2HandleDeviceChangedEvent(void *event);
     int m_xiOpCode, m_xiEventBase, m_xiErrorBase;
-#ifdef XCB_USE_XINPUT22
-    void xi2ProcessTouch(void *xiDevEvent, QXcbWindow *platformWindow);
-#endif // XCB_USE_XINPUT22
 #ifndef QT_NO_TABLETEVENT
     struct TabletData {
         TabletData() : deviceId(0), pointerType(QTabletEvent::UnknownPointer),
@@ -575,11 +528,10 @@ private:
         };
         QHash<int, ValuatorClassInfo> valuatorInfo;
     };
-    bool xi2HandleTabletEvent(void *event, TabletData *tabletData, QXcbWindowEventListener *eventListener);
+    bool xi2HandleTabletEvent(void *event, TabletData *tabletData);
     void xi2ReportTabletEvent(TabletData &tabletData, void *event);
     QVector<TabletData> m_tabletData;
-    TabletData *tabletDataForDevice(int id);
-#endif // !QT_NO_TABLETEVENT
+#endif
     struct ScrollingDevice {
         ScrollingDevice() : deviceId(0), verticalIndex(0), horizontalIndex(0), orientations(0), legacyOrientations(0) { }
         int deviceId;
@@ -592,17 +544,17 @@ private:
     void updateScrollingDevice(ScrollingDevice& scrollingDevice, int num_classes, void *classes);
     void xi2HandleScrollEvent(void *event, ScrollingDevice &scrollingDevice);
     QHash<int, ScrollingDevice> m_scrollingDevices;
+#endif // XCB_USE_XINPUT2
 
+#if defined(XCB_USE_XINPUT2)
     static bool xi2GetValuatorValueIfSet(void *event, int valuatorNum, double *value);
-    static void xi2PrepareXIGenericDeviceEvent(xcb_ge_event_t *event);
+    static bool xi2PrepareXIGenericDeviceEvent(xcb_ge_event_t *event, int opCode);
 #endif
 
     xcb_connection_t *m_connection;
     const xcb_setup_t *m_setup;
     bool m_canGrabServer;
-    xcb_visualid_t m_defaultVisualId;
 
-    QList<QXcbVirtualDesktop *> m_virtualDesktops;
     QList<QXcbScreen *> m_screens;
     int m_primaryScreenNumber;
 
@@ -628,8 +580,13 @@ private:
 #endif
     QXcbEventReader *m_reader;
 #if defined(XCB_USE_XINPUT2)
+    QHash<int, QWindowSystemInterface::TouchPoint> m_touchPoints;
     QHash<int, XInput2TouchDeviceData*> m_touchDevices;
 #endif
+#if defined(XCB_USE_EGL)
+    void *m_egl_display;
+    bool m_has_egl;
+#endif
 #ifdef Q_XCB_DEBUG
     struct CallInfo {
         int sequence;
@@ -650,32 +607,26 @@ private:
     uint32_t xfixes_first_event;
     uint32_t xrandr_first_event;
     uint32_t xkb_first_event;
+    uint32_t glx_first_event;
 
-    bool has_xinerama_extension;
+    bool has_glx_extension;
     bool has_shape_extension;
     bool has_randr_extension;
     bool has_input_shape;
+    bool has_touch_without_mouse_emulation;
     bool has_xkb;
 
     Qt::MouseButtons m_buttons;
 
     QXcbWindow *m_focusWindow;
-    QXcbWindow *m_mouseGrabber;
-    QXcbWindow *m_mousePressWindow;
 
-    xcb_window_t m_clientLeader;
     QByteArray m_startupId;
     QXcbSystemTrayTracker *m_systemTrayTracker;
-    QXcbGlIntegration *m_glIntegration;
-    bool m_xiGrab;
-
-    xcb_window_t m_qtSelectionOwner;
 
     friend class QXcbEventReader;
 };
 
 #define DISPLAY_FROM_XCB(object) ((Display *)(object->connection()->xlib_display()))
-#define CREATE_VISUALINFO_FROM_DEFAULT_VISUALID(object) ((XVisualInfo *)(object->connection()->createVisualInfoForDefaultVisualId()))
 
 template<typename T>
 xcb_generic_event_t *QXcbConnection::checkEvent(T &checker)
@@ -717,7 +668,12 @@ cookie_t q_xcb_call_template(const cookie_t &cookie, QXcbConnection *connection,
 #else
 #define Q_XCB_CALL(x) x
 #define Q_XCB_CALL2(x, connection) x
-#define Q_XCB_NOOP(c) (void)c;
+#define Q_XCB_NOOP(c)
+#endif
+
+
+#if defined(XCB_USE_EGL)
+#define EGL_DISPLAY_FROM_XCB(object) ((EGLDisplay)(object->connection()->egl_display()))
 #endif
 
 QT_END_NAMESPACE
